#include "../df_active_mode.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN int DF_API_CALL DFCStartActiveConnection( char *networkAddress, int listenPort, int deviceAddress, int timeout, int aliveTimeout, int maxConnections, int infoFlags, int unused_setTo0, int *errorNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s", __func__);

    // zulaessige Bereiche pruefen
    if ((networkAddress == NULL) || inet_addr(networkAddress) == INADDR_NONE || 
        (errorNumber == NULL) || maxConnections < 1 || maxConnections > 140) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t(%i)", (networkAddress != NULL) ? networkAddress : "", listenPort, deviceAddress, timeout, aliveTimeout, maxConnections, infoFlags, unused_setTo0,  IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // 2012_07_09 (BO) V04.02.03: deviceID fix auf 0 gestellt.
    int iRes = theApp.StartActiveConnection(networkAddress, listenPort, 0, timeout, aliveTimeout, maxConnections, infoFlags, unused_setTo0, errorNumber);

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t%i\t(%i)", networkAddress, listenPort, deviceAddress, timeout, aliveTimeout, maxConnections, infoFlags, unused_setTo0,  *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCStopActiveConnection( int *errorNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s", __func__);

    // zulaessige Bereiche pruefen
    if (errorNumber == NULL) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = theApp.StopActiveConnection();

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetFirstActiveConnectionId()
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        return 0;
    }

    pChannel->logBegin("%s", __func__);

    int i, iRes = -1;//, connected = 0;
    // Besteht die Aktive Verbindung
    if (theApp.IsActiveConnection()) 
    {
        // Erste aktive Verbindung ermitteln
        for (i=1; i < ACTIVE_CHANNEL_UPPER_BOUND; i++) 
        {
            if ((theApp.m_channelArray[i] != NULL) && (theApp.m_channelArray[i]->isActive())) 
            {
                iRes = i;
                break;
            }
        }
    }

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetFirstActiveChannelID()
{
    return DFCGetFirstActiveConnectionId();
}

DF_API_EXTERN int DF_API_CALL DFCGetNextActiveConnectionId( int previousId )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, previousId);

    // Naechste aktive Verbindung ermitteln
    int i, iRes = -1;
    // Besteht die Aktive Verbindung
    if (theApp.IsActiveConnection()) 
    {
        for (i=previousId+1; i < ACTIVE_CHANNEL_UPPER_BOUND; i++) 
        {
            if ((theApp.m_channelArray[i] != NULL) && (theApp.m_channelArray[i]->isActive())) 
            {
                iRes = i;
                break;
            }
        }
    }

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetNextActiveChannelID( int previousId )
{
    return DFCGetNextActiveConnectionId( previousId );
}

DF_API_EXTERN int DF_API_CALL DFCGetInfoActiveChannel( int connectionId, char *value, int *valueLength )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        return 0;
    }

    int iRes = 0;
    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((value == NULL) || (valueLength == NULL)) 
    {
        pChannel->logEnd("\t\t(%i)", 0);
        return 0;
    }

    // Besteht die Aktive Verbindung
    if (theApp.IsActiveConnection()) 
    {
        if ((theApp.m_channelArray[connectionId] != NULL) && (theApp.m_channelArray[connectionId]->isActive())) 
        {
            Channel4 &channel = theApp.m_channelArray[connectionId]->_channel4;
            // Aktuellen Infostring aus Kanal übernehmen.
            if ( channel.getInfoActiveChannel( value, valueLength ) )
                iRes = 1;
        }
    }

    pChannel->logEnd("\t%s\t%i\t(%i)", value, *valueLength, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetRecordAvailable( int enabled )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, enabled);

    int iRes = theApp.SetRecordAvailable(enabled);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCRecordAvailable( int *connectionId, int *deviceAddress, char *infoText, int *infoTextLength, int *errorNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s", __func__);

    // zulaessige Bereiche pruefen
    if ((connectionId == NULL) || (deviceAddress == NULL) 
        || (infoTextLength == NULL) || (infoText == NULL && *infoTextLength > 0) || (errorNumber == NULL) ) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = 0;
    // Ist das Melden von Datenaetzen aktiviert?
    if (theApp.IsNotifyRecord()) 
    {
        iRes = theApp.RecordAvailable(connectionId, deviceAddress);
        if (iRes == IV_ERROR_SUCCESS) 
        {
            Channel4 &channel = theApp.m_channelArray[*connectionId]->_channel4;
            // Aktuellen Infostring aus Kanal übernehmen.
            if ( *infoTextLength > 0 && !channel.getInfoActiveChannel( infoText, infoTextLength ) )
            {
                *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
                iRes = 0;
            }
            else
            {
                *errorNumber = IV_ERROR_SUCCESS;
                iRes = 1;
            }
        } 
        else if (iRes > IV_ERROR_SUCCESS) 
        {
            *errorNumber = IV_ERROR_PROCESSING;
            iRes = 0;
        }
        else 
        {
            *errorNumber = -iRes;
            iRes = 0;
        }
    } 
    else 
    {
        *errorNumber = IV_ERROR_AC_NO_NOTIFY_RECORD;
    }

    pChannel->logEnd("\t%i\t%i\t%s\t%i\t%i\t(%i)", *connectionId, *deviceAddress, infoText, *infoTextLength, *errorNumber, iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCBindDeviceToChannel( int connectionId, int deviceType, int deviceSerial, char *deviceIp, int *errorNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL) 
    {
        return 0;
    }

    int iRes = 0;
    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceType, deviceSerial);

    // Liegen die Werte im gueltigen Bereich.
    if (connectionId < 1 || connectionId >= ACTIVE_CHANNEL_LOWER_BOUND || ((deviceType != -1) && (deviceType < 0 || deviceType >= IV_VALUE_TYPE_DEVICE_COUNT)) ||
        (deviceIp != NULL && strlen(deviceIp) && inet_addr(deviceIp) == INADDR_NONE) || errorNumber == NULL) 
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Es gibt 4 moegliche mappings
    // 1. Loeschen eines bestehenden mappings
    // 2. map channelID => deviceType + deviceSerial
    // 3. map channelID => deviceType + deviceSerial + deviceIP
    // 4. map channelID => deviceIP
    ChannelBridge *channel = theApp.GetAEChannel(connectionId);
    if (channel->isBind() && deviceType == -1 && (deviceIp == 0 || !strlen(deviceIp))) 
    {
        // Bindungsstruktur aufbauen und zuweisen
        StructBindInfo bi;
        memset(&bi, 0, sizeof(StructBindInfo));
        channel->setBind(bi);
    } 
    else if ((deviceType > -1 && deviceSerial < 1) || (deviceType == -1 && (deviceIp == NULL || !strlen(deviceIp))) ) 
    {
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    } 
    else 
    {
        // Der Kanal darf nicht bereits gebunden oder geoeffnet sein.
        if (channel->isBind() || channel->isOpen())
        {
            *errorNumber = IV_ERROR_CHANNEL_USED;
            pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", *errorNumber, 0);
            return 0;
        }

        // Bindungsstruktur aufbauen und zuweisen
        StructBindInfo bi;
        memset(&bi, 0, sizeof(StructBindInfo));
        bi.state = 1;
        bi.typeDevice = deviceType;
        bi.serial = deviceSerial;
        if ( deviceIp != 0 )
            strncpy( (char*)bi.ip, deviceIp, sizeof(bi.ip) - 1 );

        channel->setBind(bi);

        // Erfolgte Anbindung dem Listen-Thread signalisieren
        theApp.m_bindSignal = 1;
    }

    iRes = 1;
    pChannel->logEnd("\t%s\t%i\t(%i)", (deviceIp != NULL) ? deviceIp : "", *errorNumber, iRes);

    return iRes;
}
